//-----------------------------------------------------
//File: lcd_4bit.c
//Auth: ELECTRONIC ASSEMBLY JM
//DATE: 08-07-2012
//-----------------------------------------------------

//---Includes---
#include "sfr_r825.h"
#include "define.h"
#include "lcd_4bit.h"

#include "timer.h"

//--- module global varibles ---
#define WAITST 40
//#define Wait(x)
//#define Wait(x) asm("nop")
void Wait (unsigned char waitstates)
{
	unsigned char i=0;
	for (i=0; i<waitstates; i++)
	{
		asm("nop");
	}
}



//-----------------------------------------------------
//Func: initDispl()
//Desc: inits Display
//-----------------------------------------------------
void initDispl(void)
{
	unsigned char i;
	EN = RS = RW = 0; //All pins to low
	DATA = 0x0F;
	ddEN  = 1;
	ddRW = 1;
	ddRS = 1;
	prc2 = 1;  //remove protection of port P0 to change port direction
	ddDATA = 0xFF;

	RS = 0;
	RW = 0;
	//Make sure to get in 4-Bit Mode
	WriteNibble(0x30);
	Wait(WAITST);
	WriteNibble(0x30);
	Wait(WAITST);
	WriteNibble(0x30);
	Wait(WAITST);
	WriteNibble(0x20);

	Delay(10);

	
	//init Display
#ifdef DIP203_4
	WriteIns(0x24);	//8-Bit, RE=1
	WriteIns(0x09);	//4 line mode
	WriteIns(0x20);	//8-Bit, RE=0
	WriteIns(0x0F);	//Display on, cursor on, cursor blink
	WriteIns(0x01);	//clear display
	WriteIns(0x06);	//return home
#endif


#ifdef DIP203_6	
	WriteIns(0x20);	//4-Bit, RE=0
	WriteIns(0x06);	//cursor auto increment
	WriteIns(0x26);	//4-Bit, RE=1, BE=1
	WriteIns(0x09);	//4 line mode

	WriteIns(0x40);
	for (i=0; i<16; i++)
		WriteData(0x00);

	WriteIns(0x20);	//4-Bit, RE=0
	WriteIns(0x0F);	//Display on, cursor on, cursor blink
	WriteIns(0x01);	//clear display
	WriteIns(0x06);	//return home
#endif
	
}

//-----------------------------------------------------
//Func: WriteChar(character)
//Desc: sends a single character to display
//-----------------------------------------------------
void WriteChar (char character)
{
	WriteData(character);
}

//-----------------------------------------------------
//Func: WriteString(string)
//Desc: sends a string to display, must be 0 terminated
//-----------------------------------------------------
void WriteString(_far char * string)
{
	do
	{
		WriteData(*string++);
	}
	while(*string);
}

//-----------------------------------------------------
//Func: SetSymb(addresse, data)
//Desc: shows a symbol, if databit 8/6 clear/set all
//-----------------------------------------------------
void SetSymb (char add, char data)
{
	unsigned char i=0;
	unsigned char cursorpos = GETCURSORADDR();
	WriteIns(0x26);
	WriteIns(add);
	if(data&0x80)
	{
		for (i=0; i<16; i++)
			WriteData(0x00);
	}
	else if(data&0x20)
	{
		data&=0x5F;
		for (i=0; i<16; i++)
			WriteData(data);
	}
	else
		WriteData(data);
	WriteIns(0x20);
	SetPostion(cursorpos);

}

//-----------------------------------------------------
//Func: SetPostion(postion)
//Desc: set postion
//-----------------------------------------------------
void SetPostion(char pos)
{
	WriteIns(LCD_HOME_L1+pos);
}

//-----------------------------------------------------
//Func: DisplayOnOff(control)
//Desc: use definitions of header file to set display
//-----------------------------------------------------
void DisplayOnOff(char data)
{
	WriteIns(0x08+data);
}

//-----------------------------------------------------
//Func: DefineCharacter(memory postion, character data)
//Desc: stores an own defined character
//-----------------------------------------------------
void DefineCharacter(unsigned char postion, unsigned char *data)
{
	unsigned char i=0;
	WriteIns(0x40+8*postion);

	for (i=0; i<8; i++)
	{
		WriteData(data[i]);
	}
	SetPostion(LINE1);
}

//-----------------------------------------------------
//Func: ClrDisplay
//Desc: Clears entire Display content and set home pos
//-----------------------------------------------------
void ClrDisplay(void)
{
	WriteIns(0x01);
	SetPostion(LINE1);
}

//-----------------------------------------------------
//Func: WriteIns(instruction)
//Desc: sends instruction to display
//-----------------------------------------------------
static void WriteIns(char ins)
{
	CheckBusy();
	RS = 0;
	RW = 0;
	WriteNibble(ins); 	//High nibble
	Wait(WAITST);
	WriteNibble(ins<<4); 		//low nibble
}

//-----------------------------------------------------
//Func: WriteData(data)
//Desc: sends data to display
//-----------------------------------------------------
static void WriteData(char data)
{
	CheckBusy();
	RS = 1;
	RW = 0;
	WriteNibble(data); 	//High nibble
	Wait(WAITST);
	WriteNibble(data<<4); 		//low nibble

}

//-----------------------------------------------------
//Func: WriteNibble(data)
//Desc: sends 4 bit to display, use high 4 bits of Byte
//-----------------------------------------------------
static void WriteNibble(char data)
{
	DATA = data|0x0F;

	EN = 1;
	Wait(WAITST/2);
	EN = 0;
}

//-----------------------------------------------------
//Func: CheckBusy()
//Desc: checks if display is idle
//-----------------------------------------------------
unsigned char CheckBusy(void)
{
	unsigned char readData = 1;

	prc2 = 1;
	ddDATA = 0x0F; // Data is input
	RS = 0;
	RW = 1;
	Wait(WAITST);
	do
	{
		EN = 1;
		Wait(WAITST/2);
		EN = 0;
		readData = DATA&0xF0;
		Wait(WAITST);

		EN = 1;
		Wait(WAITST/2);
		EN = 0;
		readData |= DATA>>4;
		Wait(WAITST);


	}while(readData&0x80); //check for busyflag

	prc2 = 1;  //remove protection of port P0 to change port direction
	ddDATA = 0xFF;
	return readData;
}